import numpy as np
from savant_rs.utils import *
from timeit import default_timer as timer

num = 10_000
dims = (1024, 1)


def bench_matrix(dtype, dims, num):
    t = timer()
    v = np.zeros(dims, dtype=dtype)
    m = None
    for _ in range(num):
        m = np_to_matrix(v)
    print(f"NP {dtype} > NALGEBRA {dtype}: {num} Time:", timer() - t)

    t = timer()
    for _ in range(num):
        v = matrix_to_np(m)

    print(f"NALGEBRA {dtype} > NP {dtype}: {num} Time:", timer() - t)


def bench_ndarray(dtype, dims, num):
    t = timer()
    v = np.zeros(dims, dtype=dtype)
    m = None
    for _ in range(num):
        m = np_to_ndarray(v)
    print(f"NP {dtype} > NDARRAY {dtype}: {num} Time:", timer() - t)

    t = timer()
    for _ in range(num):
        v = ndarray_to_np(m)

    print(f"NDARRAY {dtype} > NP {dtype}: {num} Time:", timer() - t)


print("Bench Nalgebra Matrix")
for dt in ['uint8', 'int8', 'uint16', 'int16', 'uint32', 'int32', 'uint64', 'int64', 'float32', 'float64']:
    bench_matrix(dt, dims, num)

print("Bench NDArray")
for dt in ['uint8', 'int8', 'uint16', 'int16', 'uint32', 'int32', 'uint64', 'int64', 'float32', 'float64']:
    bench_ndarray(dt, dims, num)


arr = np.zeros((3, 4), dtype='float32')
print(np_to_ndarray(arr))



from savant_rs.video_object_query import FloatExpression as FE, \
    IntExpression as IE, \
    StringExpression as SE, \
    Query as Q

and_ = Q.and_
or_ = Q.or_
not_ = Q.not_


gt = IE.gt
lt = IE.lt
eq = IE.eq
fgt = FE.gt

q = and_(
    Q.creator(SE.one_of('savant', 'deepstream')),
    Q.label(SE.one_of('person', 'cyclist')),
    and_(
        or_(
            not_(Q.parent_defined()),
            or_(
                Q.parent_id(IE.one_of(0, 1, 2)),
                Q.parent_id(gt(10))
            )
        )
    ),
    Q.attributes_jmes_query("[?(name=='test' && creator=='test')]"),
    Q.confidence(FE.gt(0.5)),
    Q.box_height(FE.gt(100)),
)

print("------------------------")
print("Condensed JSON:")
print("------------------------")
print(q.json)

print("------------------------")
print("Pretty JSON:")
print("------------------------")
print(q.json_pretty)

print("------------------------")
print("YAML:")
print("------------------------")
print(q.yaml)

q2 = Q.from_json(q.json)
assert q.json == q2.json

q3 = Q.from_yaml(q.yaml)
assert q3.yaml == q.yaml




from pprint import pprint

from savant_rs.primitives import PolygonalArea, Point, Segment, IntersectionKind
from timeit import default_timer as timer

area = PolygonalArea([Point(-1, 1), Point(1, 1), Point(1, -1), Point(-1, -1)], ["up", None, "down", None])
assert area.is_self_intersecting() == False

bad_area = PolygonalArea([Point(-1, -1), Point(1, 1), Point(1, -1), Point(-1, 1)], ["up", None, "down", None])
assert bad_area.is_self_intersecting() == True

bad_area2 = PolygonalArea([Point(-1, -1), Point(1, 1), Point(1, 1), Point(-1, 1)], ["up", None, "down", None])
assert bad_area2.is_self_intersecting() == True

good_area2 = PolygonalArea([Point(-1, -1), Point(1, 1), Point(-1, 1)], ["up", None, "down"])
assert good_area2.is_self_intersecting() == False

crosses_031 = Segment(Point(-2, 1), Point(2, 1))
crosses_013 = Segment(Point(2, 1), Point(-2, 1))
crosses_31 = Segment(Point(-2, 0), Point(2, 0))
crosses_20 = Segment(Point(0, -2), Point(0, 2))
leaves_vertex = Segment(Point(0, 0), Point(2, 2))
crosses_vertices = Segment(Point(-2, -2), Point(2, 2))
crosses_whole_edge = Segment(Point(-2, 1), Point(2, 1))
enters_vertex = Segment(Point(2, 2), Point(0, 0))
outside = Segment(Point(-2, 2), Point(2, 2))
inside = Segment(Point(-0.5, -0.5), Point(0.5, 0.5))

l = [crosses_31, crosses_20, crosses_031, crosses_013, leaves_vertex, crosses_vertices, crosses_whole_edge, enters_vertex, outside, inside]

t = timer()

res = None
for _ in range(10_000):
    res = area.crossed_by_segments(l)

print("Spent", timer() - t)
pprint(list(zip(l, res)))

r = res[0]
assert (r.kind == IntersectionKind.Cross)
assert (r.edges == [(3, None), (1, None)])

r = res[1]
assert (r.kind == IntersectionKind.Cross)
assert (r.edges == [(2, "down"), (0, "up")])

r = res[2]
assert (r.kind == IntersectionKind.Cross)
assert (r.edges == [(0, "up"), (3, None), (1, None)])

r = res[3]
assert (r.kind == IntersectionKind.Cross)
assert (r.edges == [(0, "up"), (1, None), (3, None)])


from timeit import default_timer as timer

import savant_rs

w = savant_rs.CopyWrapper(1)

t = timer()


for _ in range(1_000_000):
    i = w.get()
    i.inc()

print(timer() - t)
print(w.get().val())